@@ -133,13 +133,18 @@ listener_active_changed_cb (GSListener *listener,
                             GSMonitor  *monitor)
 {
         gboolean res;
+        gboolean idle_watch_enabled;
+
+        idle_watch_enabled = gs_watcher_get_enabled (monitor->priv->watcher);
 
         if (active) {
-                /* turn off the idleness watcher */
-                res = gs_watcher_set_active (monitor->priv->watcher, FALSE);
-                if (! res) {
-                        g_warning ("Unable to deactivate the idle watcher");
-                        return FALSE;
+                if (idle_watch_enabled) {
+                        /* turn off the idleness watcher */
+                        res = gs_watcher_set_active (monitor->priv->watcher, FALSE);
+                        if (! res) {
+                                g_warning ("Unable to deactivate the idle watcher");
+                                return FALSE;
+                        }
                 }
 
                 /* blank the screen */
@@ -170,11 +175,13 @@ listener_active_changed_cb (GSListener *listener,
                         return FALSE;
                 }
 
-                /* turn on the idleness watcher */
-                res = gs_watcher_set_active (monitor->priv->watcher, TRUE);
-                if (! res) {
-                        g_warning ("Unable to activate the idle watcher");
-                        return FALSE;
+                if (idle_watch_enabled) {
+                        /* turn on the idleness watcher */
+                        res = gs_watcher_set_active (monitor->priv->watcher, TRUE);
+                        if (! res) {
+                                g_warning ("Unable to activate the idle watcher");
+                                return FALSE;
+                        }
                 }
 
                 /* disable power management */
@@ -235,6 +242,9 @@ static void
 _gs_monitor_update_from_prefs (GSMonitor *monitor,
                                GSPrefs   *prefs)
 {
+        gboolean idle_detection_enabled;
+        gboolean idle_detection_active;
+
         gs_manager_set_lock_enabled (monitor->priv->manager, monitor->priv->prefs->lock_enabled);
         gs_manager_set_lock_timeout (monitor->priv->manager, monitor->priv->prefs->lock_timeout);
         gs_manager_set_logout_enabled (monitor->priv->manager, monitor->priv->prefs->logout_enabled);
@@ -245,9 +255,18 @@ _gs_monitor_update_from_prefs (GSMonitor *monitor,
         gs_manager_set_mode (monitor->priv->manager, monitor->priv->prefs->mode);
         gs_manager_set_themes (monitor->priv->manager, monitor->priv->prefs->themes);
 
+        idle_detection_enabled = (monitor->priv->prefs->mode != GS_MODE_DONT_BLANK);
         gs_watcher_set_timeout (monitor->priv->watcher, monitor->priv->prefs->timeout);
-        gs_watcher_set_active (monitor->priv->watcher,
-                               monitor->priv->prefs->mode != GS_MODE_DONT_BLANK);
+        gs_watcher_set_enabled (monitor->priv->watcher, idle_detection_enabled);
+
+        /* in the case where idle detection is reenabled we may need to
+           activate the watcher too */
+        idle_detection_active = gs_watcher_get_active (monitor->priv->watcher);
+        if (! gs_manager_is_active (monitor->priv->manager)
+            && ! idle_detection_active
+            && idle_detection_enabled) {
+                gs_watcher_set_active (monitor->priv->watcher, TRUE);
+        }
 
         gs_power_set_timeouts (monitor->priv->power,
                                monitor->priv->prefs->dpms_standby,
@@ -258,16 +277,19 @@ _gs_monitor_update_from_prefs (GSMonitor *monitor,
 }
 
 static void
-gs_monitor_init (GSMonitor *monitor)
+disconnect_listener_signals (GSMonitor *monitor)
 {
+        g_signal_handlers_disconnect_by_func (monitor->priv->listener, listener_lock_cb, monitor);
+        g_signal_handlers_disconnect_by_func (monitor->priv->listener, listener_quit_cb, monitor);
+        g_signal_handlers_disconnect_by_func (monitor->priv->listener, listener_cycle_cb, monitor);
+        g_signal_handlers_disconnect_by_func (monitor->priv->listener, listener_active_changed_cb, monitor);
+        g_signal_handlers_disconnect_by_func (monitor->priv->listener, listener_throttled_changed_cb, monitor);
+        g_signal_handlers_disconnect_by_func (monitor->priv->listener, listener_poke_cb, monitor);
+}
 
-        monitor->priv = GS_MONITOR_GET_PRIVATE (monitor);
-
-        monitor->priv->prefs = gs_prefs_new ();
-        g_signal_connect_swapped (monitor->priv->prefs, "changed",
-                                  G_CALLBACK (_gs_monitor_update_from_prefs), monitor);
-
-        monitor->priv->listener = gs_listener_new ();
+static void
+connect_listener_signals (GSMonitor *monitor)
+{
         g_signal_connect (monitor->priv->listener, "lock",
                           G_CALLBACK (listener_lock_cb), monitor);
         g_signal_connect (monitor->priv->listener, "quit",
@@ -280,23 +302,85 @@ gs_monitor_init (GSMonitor *monitor)
                           G_CALLBACK (listener_throttled_changed_cb), monitor);
         g_signal_connect (monitor->priv->listener, "poke",
                           G_CALLBACK (listener_poke_cb), monitor);
+}
 
-        monitor->priv->watcher = gs_watcher_new (monitor->priv->prefs->timeout);
+static void
+disconnect_watcher_signals (GSMonitor *monitor)
+{
+        g_signal_handlers_disconnect_by_func (monitor->priv->watcher, watcher_idle_cb, monitor);
+}
+
+static void
+connect_watcher_signals (GSMonitor *monitor)
+{
         g_signal_connect (monitor->priv->watcher, "idle",
                           G_CALLBACK (watcher_idle_cb), monitor);
+}
 
-        monitor->priv->manager = gs_manager_new ();
+static void
+disconnect_manager_signals (GSMonitor *monitor)
+{
+        g_signal_handlers_disconnect_by_func (monitor->priv->manager, manager_activated_cb, monitor);
+        g_signal_handlers_disconnect_by_func (monitor->priv->manager, manager_deactivated_cb, monitor);
+}
+
+static void
+connect_manager_signals (GSMonitor *monitor)
+{
         g_signal_connect (monitor->priv->manager, "activated",
                           G_CALLBACK (manager_activated_cb), monitor);
         g_signal_connect (monitor->priv->manager, "deactivated",
                           G_CALLBACK (manager_deactivated_cb), monitor);
+}
 
-        monitor->priv->power = gs_power_new ();
+static void
+disconnect_power_signals (GSMonitor *monitor)
+{
+        g_signal_handlers_disconnect_by_func (monitor->priv->power, power_changed_cb, monitor);
+}
+
+static void
+connect_power_signals (GSMonitor *monitor)
+{
         g_signal_connect (monitor->priv->power, "changed",
                           G_CALLBACK (power_changed_cb), monitor);
+}
+
+static void
+disconnect_prefs_signals (GSMonitor *monitor)
+{
+        g_signal_handlers_disconnect_by_func (monitor->priv->prefs, _gs_monitor_update_from_prefs, monitor);
+}
+
+static void
+connect_prefs_signals (GSMonitor *monitor)
+{
+        g_signal_connect_swapped (monitor->priv->prefs, "changed",
+                                  G_CALLBACK (_gs_monitor_update_from_prefs), monitor);
+}
+
+static void
+gs_monitor_init (GSMonitor *monitor)
+{
+
+        monitor->priv = GS_MONITOR_GET_PRIVATE (monitor);
+
+        monitor->priv->prefs = gs_prefs_new ();
+        connect_prefs_signals (monitor);
+
+        monitor->priv->listener = gs_listener_new ();
+        connect_listener_signals (monitor);
+
+        monitor->priv->watcher = gs_watcher_new (monitor->priv->prefs->timeout);
+        connect_watcher_signals (monitor);
+
+        monitor->priv->manager = gs_manager_new ();
+        connect_manager_signals (monitor);
+
+        monitor->priv->power = gs_power_new ();
+        connect_power_signals (monitor);
 
         _gs_monitor_update_from_prefs (monitor, monitor->priv->prefs);
-
 }
 
 static void
@@ -311,9 +395,16 @@ gs_monitor_finalize (GObject *object)
 
         g_return_if_fail (monitor->priv != NULL);
 
+        disconnect_watcher_signals (monitor);
+        disconnect_listener_signals (monitor);
+        disconnect_manager_signals (monitor);
+        disconnect_power_signals (monitor);
+        disconnect_prefs_signals (monitor);
+
         g_object_unref (monitor->priv->watcher);
         g_object_unref (monitor->priv->listener);
         g_object_unref (monitor->priv->manager);
+        g_object_unref (monitor->priv->power);
         g_object_unref (monitor->priv->prefs);
 
         G_OBJECT_CLASS (parent_class)->finalize (object);
